%% BYOM, ERA-special with simple-compound model: byom_pred_epx.m
%
% *Table of contents*

%% About
% * Author     : Tjalling Jager
% * Date       : November 2022
% * Web support: <http://www.debtox.info/byom.html>
% * Back to index <walkthrough_debtox2019.html>
%
% BYOM is a General framework for simulating model systems in terms of
% ordinary differential equations (ODEs). The model itself needs to be
% specified in <derivatives.html derivatives.m>, and <call_deri.html
% call_deri.m> may need to be modified to the particular problem as well.
% The files in the engine directory are needed for fitting and plotting.
% Results are shown on screen but also saved to a log file (results.out).
%
% *The model:* simple DEBtox model for toxicants, based on DEBkiss and
% formulated in compound parameters. The model includes flexible modules
% for toxicokinetics/damage dynamics and toxic effects. The DEBkiss e-book
% (see <http://www.debtox.info/book_debkiss.html>) provides a description
% of the model, as well as the publication of Jager in Ecological
% Modelling: <https://doi.org/10.1016/j.ecolmodel.2019.108904>.
%
% *This script:* This script demonstrates the use of the parameter-space
% explorer from the openGUTS project (see <http://www.openguts.info/>) in
% making predictions for FOCUS profiles. This script works generically, as
% all species- and compound-specific information is loaded from the MAT
% file. Furthermore, it is now even largely model independent, as glo and
% X0mat are also saved during the calibration, and loaded here. All
% DEB-based TKTD analyses should work with this script.
% 
%  Copyright (c) 2012-2022, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(1) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 0; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% Data are entered in matrix form, time in rows, scenarios (exposure
% concentrations) in columns. First column are the exposure times, first
% row are the concentrations or scenario numbers. The number in the top
% left of the matrix indicates how to calculate the likelihood:

% No data for simulations!

%% Call the Matlab GUI open-file element to load profile and MAT file

[conf_type,fname_prof,par] = select_pred([2 1 1]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters, as well as all settings used to generate the MAT
% file (in glo) apart from the exposure profile (which we'll redefine
% anyway). This only works when the MAT file was generated by BYOM v.6.0
% BETA 7 or newer as older versions won't save glo. When using an old MAT
% file (pre BYOM v6), make sure to set glo.Tbp to the correct value if it
% needs to be >0.

% The parameter structure <par> is loaded from the selected MAT file. Even
% though it does not need to be used to call other functions, it must be
% specified for the code to run error free.

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios. All identifiers and initial values, as part of
% the matrix X0mat, are also loaded by select_pred already. Feel free to
% overwrite them with other values if needed.
% 
% NOTE: the glo.locD etc. have all been loaded from the MAT file by select_pred.

%% Initial values for the model parameters
% All parameters have been loaded from file using select_pred. All global
% parameters, as part of the structure glo, are also loaded by
% select_pred already. Feel free to overwrite them with other values.
  
% glo.len = 2;  % switch to fit length 1) with shrinking, 2) without shrinking (used in call_deri.m)

%% Time vector and labels for plots
% Specify what to plot. If time vector glo.t is not specified, a default is
% constructed, based on the data set.

% All plot labels have been loaded from file using select_pred.
% Vector glo.t is already defined in select_pred from the selected profile.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Calculations and plotting
% Here, the function is called that will do the calculation and the plotting.
% Options for the plotting can be set using opt_plot (see prelim_checks.m).
% Options for the optimsation routine can be set using opt_optim. Options
% for the ODE solver are part of the global glo. 
% 
% NOTE: for this package, the options useode and eventson in glo will not
% be functional: the ODE solver is always used, and the events function as
% well.
 
% -------------------------------------------------------------------------
% Configurations for the ODE solver
glo.stiff = [0 3]; % ODE solver 0) ode45 (standard), 1) ode113 (moderately stiff), 2) ode15s (stiff)
% Second argument is for default sloppy (0), normally tight (1), tighter
% (2), or very tight (3) tolerances. Use 1 for quick analyses, but check
% with 3 to see if there is a difference! Especially for time-varying
% exposure, there can be large differences between the settings!
glo.break_time = 0; % break time vector up for ODE solver (1) or don't (0)
% NOTE: for FOCUS profiles, breaking the time vector does not help accuracy
% much, but it does slow down the calculations a lot.
% -------------------------------------------------------------------------

% No need for optimisation or standard plotting here.

%% Calculate ECx for all endpoints
% The function calc_ecx will calculate ECx values, based on the parameter
% values (and a sample from parameter space to make CIs). The ECx is
% calculated assuming constant exposure (which is in its definition) at the
% time points requested.

opt_conf.type    = 0; % conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
opt_conf.lim_set = 2; % use limited set of n_lim points (1) or outer hull (2, not for Bayes) to create CIs
opt_ecx.statsup  = []; % states to suppress from the calculations (e.g., glo.locS)

% calc_ecx(par,[1 5 10 15 21],opt_ecx,opt_conf);
% % Second input is a vector with time points on which ECx is calculated (may
% % extend beyond the time vector of the data set).

% return

%% Analyse an exposure profile to check safety margins
% The functions below analyse the impacts of an exposure profile, given a
% certain exposure profile (e.g., from FOCUS). First, a 'moving
% time-window' analysis is done, with a fixed window of 21 days, and
% several fixed MFs. This will be a relatively fast and practical way to
% see if a certain critical safety margin is available. Next, the EPx can
% be calculated exactly.
% 
% Note that the functions calc_epx and calc_epx_window use the options
% structure opt_ecx. This option structure, by default, makes sure that the
% parameter called _hb_ is set to zero in the analysis (both in the
% best-fit parameter set and in the sample).

% opt_conf.type     = conf_type; % make intervals from 1) slice sampler, 2) likelihood region, 3) parspace explorer
opt_conf.type     = 0; % make intervals from 1) slice sampler, 2) likelihood region, 3) parspace explorer
opt_conf.lim_set  = 1; % use limited set of n_lim points (1) or outer hull (2, not for Bayes) to create CIs
% opt_conf.n_lim    = 100; % size of limited set (likelihood-region and parspace only)
% opt_ecx.statsup   = [glo.locL]; % states to suppress from the calculations (e.g., glo.locL)
% opt_ecx.id_sel    = [0 1 0]; % scenario to use from X0mat, scenario identifier, flag for ECx to use scenarios rather than concentrations
opt_ecx.rob_win   = 0; % set to 1 to use robust EPx calculation, rather than with fzero
% Note: robust EPx calculation calculates EPx at the given steps only and
% then interpolates. This is not very precise, but with carefully selected
% steps it will be the lowest. In rather extreme cases, there will be more
% than one EPx for a specific window and a specific endpoint (when there
% are effects on growth, and feedbacks affecting k_d; a warning will be
% given). The 'regular' method may end up in either EPx (or produces an
% error), while 'robust' has a far better chance to yield the lowest.
% Robust is only used for EPx calculation, not for effect windows with
% fixed MFs.

opt_ecx.start_neg = 1; % set to 1 to start the moving window at minus window width
opt_ecx.prune_win = 0; % set to 1 to prune the windows to keep the interesting ones
% Note: this setting screens all windows to find the window with highest
% minimum concentration. Any window whose maximum is lower than this value
% can be ignored. This is not guaranteed to work when there
% are effects on growth, and feedbacks affecting k_d (a warning will be
% given).
Twin = 21; % length of the time window (one element)
opt_ecx.Tstep    = 1; % stepsize or resolution of the time window (default 1 day)
opt_ecx.calc_int = 0;  % integrate survival and repro into 1) RGR, or 2) survival-corrected repro (experimental!)
opt_ecx.showprof = 1;  % set to 1 to show exposure profile in plots at top row

% % Force EPx calculations to use certain f
% par.f(1) = 0.5;
% opt_conf.use_par_out = 1; % set to 1 to use par as entered into the function for CIs, rather than from saved set

% -------------------------------------------------------------------------
% First, calculate effect (at end of each time window) for a range of fixed
% multiplication factors (MF).
opt_ecx.mf_range  = [3 10 30 100 300 1000]; % range for fixed MFs to make plots with for calc_effect_window
calc_effect_window(par,fname_prof,Twin,opt_ecx,opt_conf);
% -------------------------------------------------------------------------
% % We can also calculate explicit EPx values for each window. This will be
% % considerably slower (especially when making CIs). 
% opt_ecx.Feff = [0.10]; % effect levels (>0 en <1), x/100 in ECx/EPx
% [MinColl,~,ind_traits] = calc_epx_window(par,fname_prof,Twin,opt_ecx,opt_conf); 
% -------------------------------------------------------------------------

return

% Continue with a more detailed analysis of one window, including CI.
% MinColl contains all output, including most sensitive EPx and time point
% at which it occurs ... so we can automatically extract that time point.
[~,ind_min] = min(MinColl{opt_ecx.Feff==0.1}(:,2)); % find where lowest EP10 is 
Tstart = MinColl{opt_ecx.Feff==0.1}(ind_min,3); % start time for window with lowest EP10
% [~,ind_min] = min(MinColl{opt_ecx.Feff==0.5}(:,2)); % find where lowest EP50 is 
% Tstart = MinColl{opt_ecx.Feff==0.5}(ind_min,3); % start time for window with lowest EP50
switch ind_traits(ind_min) % suppress states that are NOT the most sensitive ones
    case glo.locS
        opt_ecx.statsup = [glo.locL;glo.locR]; % states to suppress from the calculations (e.g., glo.locL)
    case glo.locL
        opt_ecx.statsup = [glo.locS;glo.locR]; % states to suppress from the calculations (e.g., glo.locL)
    case glo.locR
        opt_ecx.statsup = [glo.locL;glo.locS]; % states to suppress from the calculations (e.g., glo.locL)
end
% % Note: alternatively, you can set Tstart manually to the starting point of
% % the window of choice 
% Tstart = 25;

opt_conf.type     = conf_type; % make intervals from 1) slice sampler, 2) likelihood region, 3) parspace explorer
opt_tktd.addzero  = 1; % set to 1 to always add a concentration zero to X0mat
opt_tktd.min      = 1; % set to 1 to show a dotted line for the control (lowest) treatment
opt_tktd.notitle  = 1; % set to 1 to suppress titles above plots
opt_tktd.max_exp  = 0; % set to 1 to maximise exposure/damage plots on exposure rather than damage
% Note: for sub-lethal endpoints here, the dotted line for the control is
% handy. The overall title can be suppressed as it may block the titles of
% the sub-panels.

opt_ecx.batch_epx = 0; % when set to 1 use batch mode (no output to screen)
opt_ecx.Feff      = [0.10]; % effect levels (>0 en <1), x/100 in ECx/EPx

Trng = [Tstart Tstart+Twin]; % time range from the profile as specified in fname_prof to calculate EPx
% Note: the Trng has 2 elements, for the start and end time of the window

% Use the following function to generate EPx. 
calc_epx(par,fname_prof,Trng,opt_ecx,opt_conf,opt_tktd); 
% leave opt_tktd empty to NOT make a plot, or set opt_ecx.plot = 0
% leave Twin empty to use the entire profile as specified in fname_prof


